part of '../fl_camera.dart';

enum FlashState {
  off,
  on,

  /// only ios
  auto
}

enum CameraResolution {
  /// android QUALITY_QVGA   ios 288*352
  low,

  /// android 480*640  ios 480*640
  medium,

  /// android 720*1280  ios 720*1280
  high,

  /// android 1080*1920  ios 1080*1920
  veryHigh,

  /// android 2160*3840  ios 2160*3840
  ultraHigh,

  /// android QUALITY_HIGH  ios max
  max
}

enum CameraLensFacing { back, front, external }

class FlCameraMethodCall {
  factory FlCameraMethodCall() => _singleton ??= FlCameraMethodCall._();

  FlCameraMethodCall._();

  static FlCameraMethodCall? _singleton;

  /// 所有可用的摄像头
  /// all available Cameras
  List<CameraInfo>? _cameras;

  /// 相机消息通道
  /// Camera message channel
  late FlCameraEvent _cameraEvent;

  /// 相机配置信息
  /// Camera configuration information
  FlCameraOptions? _cameraOptions;

  MethodChannel _channel = _flCameraChannel;

  FlCameraEvent get cameraEvent => _cameraEvent;

  MethodChannel get channel => _channel;

  FlCameraOptions? get cameraOptions => _cameraOptions;

  List<CameraInfo>? get cameras => _cameras;

  /// 第一步 先设置 channel
  /// The first step is to set the channel
  void setMethodChannel(MethodChannel channel) {
    _channel = channel;
    _cameraEvent = FlCameraEvent();
    _cameraEvent.setMethodChannel(_channel);
  }

  /// 第二步 获取可用摄像机
  /// Step 2 get available Cameras
  Future<List<CameraInfo>?> availableCameras() async {
    try {
      final List<Map<dynamic, dynamic>>? cameras = await _channel
          .invokeListMethod<Map<dynamic, dynamic>>('availableCameras');
      if (cameras == null) return <CameraInfo>[];
      _cameras = cameras
          .map((Map<dynamic, dynamic> camera) => CameraInfo(
              name: camera['name'] as String,
              lensFacing: _getCameraLensFacing(camera['lensFacing'] as String)))
          .toList();
      return _cameras;
    } on PlatformException catch (e) {
      debugPrint(e.toString());
    }
    return null;
  }

  /// 第三步 初始化相机
  /// Step 3 initialize the camera
  Future<bool> initCamera() async {
    if (!_supportPlatform) return false;
    final bool? state = await channel.invokeMethod<bool?>('initCamera');
    return state ?? false;
  }

  /// 第四步 开始预览
  /// Step 4 start Preview
  /// [camerId] 需要预览的相机 Camera ID to preview
  /// [resolution] 预览相机支持的分辨率 Preview the resolution supported by the camera
  Future<FlCameraOptions?> startPreview(String camerId,
      {CameraResolution resolution = CameraResolution.high}) async {
    if (!_supportPlatform) return null;
    final Map<dynamic, dynamic>? map = await channel
        .invokeMethod<Map<dynamic, dynamic>?>('startPreview', <String, dynamic>{
      'cameraId': camerId,
      'resolution': resolution.toString().split('.')[1]
    });
    if (map != null) {
      _cameraOptions = FlCameraOptions.fromMap(map);
      return _cameraOptions;
    }
    return null;
  }

  /// 打开/关闭 闪光灯
  /// Turn flash on / off
  Future<bool> setFlashMode(bool status) async {
    if (!_supportPlatform) return false;
    final bool? state =
        await channel.invokeMethod<bool?>('setFlashMode', status);
    return state ?? false;
  }

  /// 相机缩放
  /// Camera zoom
  Future<bool> setZoomRatio(double ratio) async {
    if (!_supportPlatform) return false;
    assert(ratio >= 1, 'ratio must be greater than or equal to 1');
    final bool? state =
        await channel.invokeMethod<bool?>('setZoomRatio', ratio);
    return state ?? false;
  }

  /// 注销相机
  /// dispose camera
  Future<bool> dispose() async {
    await cameraEvent.dispose();
    if (!_supportPlatform) return false;
    final bool? state = await channel.invokeMethod<bool?>('disposeCamera');
    return state ?? false;
  }

  CameraLensFacing _getCameraLensFacing(String lensFacing) {
    switch (lensFacing) {
      case 'back':
        return CameraLensFacing.back;
      case 'front':
        return CameraLensFacing.front;
      case 'external':
        return CameraLensFacing.external;
      default:
        return CameraLensFacing.external;
    }
  }
}

class CameraInfo {
  CameraInfo({required this.name, required this.lensFacing});

  String name;
  CameraLensFacing lensFacing;
}

class FlCameraOptions {
  FlCameraOptions.fromMap(Map<dynamic, dynamic> map) {
    textureId = map['textureId'] as int?;
    width = map['width'] as double?;
    height = map['height'] as double?;
  }

  int? textureId;
  double? width;
  double? height;

  Map<String, dynamic> toMap() => <String, dynamic>{
        'textureId': textureId,
        'width': width,
        'height': height
      };
}
